/*************************************************************************************************************
*	Title:			ShellCmd.cs
*	Description:	ShellExtension implementation
**************************************************************************************************************
*/

// Modified by Nick Sabalausky

using System;
using System.Globalization;
using System.IO;
using System.Reflection;
using System.Runtime.CompilerServices;
using System.Runtime.InteropServices;
using System.Text;
using Microsoft.Win32;
using System.Security.Permissions;


//[assembly:AssemblyKeyFile("shellcmd.snk")]			// Strong name keyfile for this assembly
[assembly:ComVisible(false)]						// Make sure that no interfaces are registered with COM

namespace ShellExt
{
	struct MenuItem
	{
		public	string	text;
		public	string	command; // I'm not really using this one
	}

	[Guid("74E8CD41-4045-4bbc-99B2-E747E115CD7E"), ComVisible(true)]
	public	class	Implementation: IShellExtInit, IContextMenu
	{
		const	string guid = "{74E8CD41-4045-4bbc-99B2-E747E115CD7E}";

		#region DllImports
		[DllImport("kernel32.dll")]
		static extern Boolean SetCurrentDirectory([MarshalAs(UnmanagedType.LPTStr)]string lpPathName);

		[DllImport("kernel32.dll")]
		static extern uint GetFileAttributes([MarshalAs(UnmanagedType.LPTStr)]string lpPathName);
		const uint FILE_ATTRIBUTE_DIRECTORY = 0x00000010;

		[DllImport("kernel32.dll")]
		static extern Boolean CreateProcess(
											string	lpApplicationName,
											string	lpCommandLine,
											uint	lpProcessAttributes,
											uint	lpThreadAttributes,
											Boolean bInheritHandles,
											uint	dwCreationFlags,
											uint	lpEnvironment,
											string	lpCurrentDirectory,
											StartupInfo lpStartupInfo,
											ProcessInformation lpProcessInformation);

		[DllImport("shell32")]
		static extern uint DragQueryFile(uint hDrop,uint iFile, StringBuilder buffer, int cch);

        [DllImport("user32")]
        static extern int MessageBox(int hWnd, string text, string caption, int type);

		[DllImport("user32")]
		static extern int InsertMenuItem(uint hmenu, uint uposition, uint uflags, ref MENUITEMINFO mii);
		#endregion

		IDataObject	m_dataObject = null;
		uint	m_hDrop = 0;

		MenuItem[] m_items;
		string scriptName;

		// From NickUtil.cs: Unfinished, need to identify all valid chars (not invalid ones).
		public static string MakeVarName(string str)
		{
			string workStr = str;
			
			workStr = workStr.Replace("\t", null);
			workStr = workStr.Replace("\n", null);
			workStr = workStr.Replace(" ", null);
			workStr = workStr.Replace("-", null);
			workStr = workStr.Replace(".", null);

			return workStr;
		}

		int	IContextMenu.QueryContextMenu(uint hMenu, uint iMenu, int idCmdFirst, int idCmdLast, uint uFlags)
		{
			int id = 1;
			if ( (uFlags & 0xf) == 0 || (uFlags & (uint)CMF.CMF_EXPLORE) != 0)
			{
				uint nselected = DragQueryFile(m_hDrop, 0xffffffff, null, 0);
				if (nselected == 1)
				{
					StringBuilder sb = new StringBuilder(1024);
					DragQueryFile(m_hDrop, 0, sb, sb.Capacity + 1);
					string directory = sb.ToString();
//					uint attr = GetFileAttributes(directory);
//					if ((attr & FILE_ATTRIBUTE_DIRECTORY) != 0)
//					{
//						// I have selected a bona-fide directory add to the menus.
						MENUITEMINFO mii = new MENUITEMINFO();
						mii.cbSize		= 48;
						mii.fMask		= (uint)MIIM.ID | (uint)MIIM.TYPE | (uint)MIIM.STATE;

//						foreach(MenuItem item in m_items)
//						{
							mii.wID			= idCmdFirst + id;
							mii.fType		= (uint)MF.STRING;
							mii.dwTypeData	= m_items[0].text;
							mii.fState		= (uint)MF.ENABLED;
							InsertMenuItem(hMenu, (uint)2, 1, ref mii);
							id++;
//						}
//						// Go and get the menus from the registry
//						if (m_items.Length > 0)
//						{
//							mii.fType		= (uint)MF.SEPARATOR;
//							mii.fState		= (uint)MF.ENABLED;
//							InsertMenuItem(hMenu, (uint)2, 1, ref mii);
//						}
//					}
				}
			}
			return id;
		}

		void	IContextMenu.GetCommandString(int idCmd, uint uFlags, int pwReserved, StringBuilder commandString, int cchMax)
		{
			switch(uFlags)
			{
			case (uint)GCS.VERB:
				commandString = new StringBuilder(m_items[idCmd - 1].command.Substring(1, cchMax-1));
				break;
			case (uint)GCS.HELPTEXT:
				commandString = new StringBuilder(m_items[idCmd - 1].command.Substring(1, cchMax));
				break;
			case (uint)GCS.VALIDATE:
				break;
			}
		}

		void	IContextMenu.InvokeCommand (IntPtr pici)
		{
			try
			{
				Type typINVOKECOMMANDINFO = Type.GetType("ShellExt.INVOKECOMMANDINFO");
				INVOKECOMMANDINFO ici = (INVOKECOMMANDINFO)Marshal.PtrToStructure(pici, typINVOKECOMMANDINFO);
				if (ici.verb - 1 <= m_items.Length)
				{
					ExecuteCommand(m_items[ici.verb - 1].command);
				}
			}
			catch(Exception e)
			{
				MessageBox(0, "Error : " + e.ToString(), "Error in EnableThisFromRun.exe", 0);
			}
		}


		int	IShellExtInit.Initialize (IntPtr /*LPCITEMIDLIST*/ pidlFolder, IntPtr /*LPDATAOBJECT*/ lpdobj, uint /*HKEY*/ hKeyProgID)
		{
			try
			{
				m_dataObject = null;
				if (lpdobj != (IntPtr)0)
				{
					// Get info about the directory
					m_dataObject = (IDataObject)Marshal.GetObjectForIUnknown(lpdobj);
					FORMATETC fmt = new FORMATETC();
					fmt.cfFormat = CLIPFORMAT.CF_HDROP;
					fmt.ptd		 = 0;
					fmt.dwAspect = DVASPECT.DVASPECT_CONTENT;
					fmt.lindex	 = -1;
					fmt.tymed	 = TYMED.TYMED_HGLOBAL;
					STGMEDIUM medium = new STGMEDIUM();
					m_dataObject.GetData(ref fmt, ref medium);
					m_hDrop = medium.hGlobal;


					/*
					 * Now retrieve the menu information from the registry
					*/
					RegistryKey sc = Registry.LocalMachine;
					sc = sc.OpenSubKey("Software\\Abscissa\\EnableThisFromRun", true);
//					if (sc.SubKeyCount > 0)
//					{
//						m_items = new MenuItem[sc.SubKeyCount];
//						int	i=0;
						scriptName = (string)sc.GetValue("ScriptFile", "");
//						foreach(string name in sc.GetSubKeyNames())
//						{
//							try
//							{
//								RegistryKey item = sc.OpenSubKey(name, true);
//								string command = (string)item.GetValue("");
//								MenuItem m = new MenuItem();
//								m.text = name;
//								m.command = command;
//								m_items[i] = m;
//								++i;
//							}
//							catch(Exception)
//							{
//							}
//						}
//					}
//					else
//					{
//						m_items = new MenuItem[0];
//					}
					m_items = new MenuItem[1];
					MenuItem m = new MenuItem();
					m.text = "Enable This From Start->Run";
					m.command = "";
					m_items[0] = m;
					
				}
			}
			catch(Exception)
			{
			}
			return 0;
		}

		private	void ExecuteCommand(string command)
		{
			StartupInfo si = new StartupInfo();
			ProcessInformation pi = new ProcessInformation();
			si.cb = 68; //sizeof(si);
			try
			{
				// Get the directory name
				StringBuilder sb = new StringBuilder(1024);
				DragQueryFile(m_hDrop, 0, sb, sb.Capacity + 1);
				string directory = sb.ToString();

				  // Nix trailing backslash (if exists)
				if(directory[directory.Length-1] == '\\')
					directory = directory.Substring(0, directory.Length-1);

				  // Get shortcut name from file (if exists), else use the dir name
				string shortcutName;
				if(File.Exists(directory + "\\_run.txt"))
				{
					using(StreamReader sr = new StreamReader(directory + "\\_run.txt"))
						shortcutName = sr.ReadLine();
				}
				else
					shortcutName = Path.GetFileNameWithoutExtension(directory);
				shortcutName = MakeVarName(shortcutName);

//				string cmd = scriptName + " /target:\"" + directory
//								+ "\" /dir:\"" + "" /* nothing for shortcut of directory */
//								+ "\" /cmd:\"" + shortcutName + "\"";
				string cmdExec;
				if(Environment.OSVersion.Platform == PlatformID.Win32NT)
					cmdExec = "cmd";
				else
					cmdExec = "command";

				string cmd = cmdExec + " /C \"" + scriptName + " /target:\"" + directory
					+ "\" /dir:\"" + "" /* nothing for shortcut of directory */
					+ "\" /cmd:\"" + shortcutName + "\"\"";
				// "C:\\EnableThisFromRun.bat /target:\"C:\\ls.bat\" /dir:\"\" /cmd:\"listit\""
				if (!CreateProcess (null, cmd, 0, 0, false, 0, 0, Path.GetDirectoryName(scriptName), si, pi)) 
				{
					using(StreamWriter sw = new StreamWriter("C:\\EnableThisFromRun.log"))
					{
						sw.WriteLine(shortcutName);
						sw.WriteLine("---");
						sw.WriteLine(directory);
						sw.WriteLine("---");
						sw.Write(cmd);
					}
					MessageBox(0, "Unable to execute script: " + cmd, "Error in EnableThisFromRun.exe", 0);
				}
			}
			catch (Exception e)
			{
				Console.WriteLine(e);
			}
		}

		[System.Runtime.InteropServices.ComRegisterFunctionAttribute()]
		static void RegisterServer(String str1)
		{
			try
			{
				RegistryKey root;
				RegistryKey rk;
				root = Registry.CurrentUser;
				rk = root.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\Explorer", true);
				rk.SetValue("DesktopProcess", 1);
				rk.Close();

				// For Winnt set me as an approved shellex
				root = Registry.LocalMachine;
				rk = root.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\Shell Extensions\\Approved", true);
				rk.SetValue(guid.ToString(), "EnableThisFromRun shell extension");
				rk.Close();

				// Set "Folder\\shellex\\ContextMenuHandlers\\EnableThisFromRun" regkey to my guid
				root = Registry.ClassesRoot;
				rk = root.CreateSubKey("Folder\\shellex\\ContextMenuHandlers\\EnableThisFromRun");
				rk.SetValue("", guid.ToString());
				rk.Close();

//				// Add setting for Command.com
				root = Registry.LocalMachine;
				rk = root.CreateSubKey("Software\\Abscissa\\EnableThisFromRun");
				rk.SetValue("ScriptFile", @"C:\EnableThisFromRun.js");
//				if (Environment.OSVersion.Platform == PlatformID.Win32NT)
//					rk.SetValue("", "cmd.exe");
//				else
//					rk.SetValue("", "command.com");

				rk.Close();
			}
			catch(Exception e)
			{
				System.Console.WriteLine(e.ToString());
			}
		}

		[System.Runtime.InteropServices.ComUnregisterFunctionAttribute()]
		static void UnregisterServer(String str1)
		{
			try
			{
				RegistryKey root;
				RegistryKey rk;

				// Remove ShellExtenstions registration
				root = Registry.LocalMachine;
				rk = root.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\Shell Extensions\\Approved", true);
				rk.DeleteValue(guid);
				rk.Close();

				// Delete ShellCmd regkey
				root = Registry.ClassesRoot;
				root.DeleteSubKey("Folder\\shellex\\ContextMenuHandlers\\EnableThisFromRun");

//				// Delete Command.com setting
				root = Registry.LocalMachine;
//				root.DeleteSubKey("Software\\Microsoft\\ShellCmd\\Command");
				root.DeleteSubKey("Software\\Abscissa\\EnableThisFromRun");
			}
			catch(Exception e)
			{
				System.Console.WriteLine(e.ToString());
			}
		}

		private static void	Usage()
		{
			System.Console.WriteLine("Syntax: EnableThisFromRun [Options]\nOptions:\n    /u             Uninstall the shell extension\n    /? or /help    Display this usage message");
		}

		public static int	Main(string[] args)
		{
			bool unregister = false;

			try
			{
				Console.WriteLine("EnableThisFromRun:\nWindows explorer extension made from modified sample code from Microsoft.\nOriginal version Copyright (C) Microsoft Corp. 2000-2002.  All rights reserved.");

				if(args.Length == 1)
				{
					string a = args[0].Replace('-', '/').ToLower(CultureInfo.InvariantCulture);
					if (a == "/?" || args[0] == "/help")
					{
						Usage();
						return 0;
					}
					else if (a == "/u")
					{
						unregister = true;
					}
					else
					{
						Console.WriteLine("Invalid option: " + args[0]);
						Usage();
						return 1;
					}
				}

				// Do my registration / alink etc.
				Assembly asm = Assembly.GetExecutingAssembly();	// Get the assembly in which I can be found
				RegistrationServices reg = new RegistrationServices();
				if(unregister)
				{
					reg.UnregisterAssembly(asm);
				}
				else
				{
					reg.RegisterAssembly(asm, AssemblyRegistrationFlags.SetCodeBase);
				}
				return 0;
			}
			catch(Exception e)
			{
				Console.WriteLine("An exception was thrown : " + e);
				return 1;
			}
		}
	}
}
